#ifndef COLORATTRIBUTE_H
#define COLORATTRIBUTE_H

#include <qcolor.h>
#include "../Interfaces/AttributeInterface.h"

/// A color in RGB.
/**
 * This class inherite from QColor, so has yet all useful functions for a color.
 */
class ColorAttribute : public AttributeInterface, public QColor{
public :
  
  /// R, G and B are changed for having a new color.
  static ColorAttribute getDifferentColor (const ColorAttribute & baseColor);

  /// The base constrcuteor, with the tree color components.
  /**
   * @param r the red component.
   * @param g the green component.
   * @param b the blue component.
   */
  ColorAttribute (int r = 0, int g = 0, int b = 0);
  
  /// The same as previous, but directly with a QColor.
  ColorAttribute (const QColor & color);
  
  virtual ~ColorAttribute () {}

  /// Modifies diffuse color of the given surfel, with its own value of red, green and blue.
  void affectSurfel (SurfelInterface * surfel, Scene * scene, bool repaint) const;
};


#endif // COLORATTRIBUTE_H
