#ifndef HYPERFUNABLE_H
#define HYPERFUNABLE_H

#include <qtextstream.h>

/// The interface for classes writable in HyperFun language.
/**
 * HyperFun version with attributes is used here for exporting somes objects (trees,
 * primitives, ...).
 */
class HyperFunable{
public:
  
  /// Write a valid HyperFun file describing the object.
  /**
   * Most of time, it writes array (<code>writeDefArray </code>), 
   * the body (<code>writeDefBody </Code>), and then, the final affectation
   * (like "my_model=...").
   *
   * @param outStream where the object must writes.
   */
  virtual void writeHFString (QTextStream & outStream) = 0;
  
  /** 
   * Write the declaration of the useful arrays for the HyperFun representation 
   * of the object.
   *
   * @param outStream where the object must writes.
   */
  virtual void writeDefArray (QTextStream & outStream) = 0;

  /**
   * Write the HyperFun main description of the object.
   *
   * @param outStream where the object must writes.
   */
  virtual void writeDefBody (QTextStream & outStream) = 0;
};

#endif //HYPERFUNABLE_H
