#include <qlabel.h>
#include <assert.h>

#include "ColorConfigurationWidget.h"

static const unsigned int COLOR_MARGIN = 30;
static const QString COLOR_TITLE = "Simple Color";
static const QColor DEFAULT_COLOR = QColor (0, 0, 0);

ColorConfigurationWidget::ColorConfigurationWidget (QWidget * parent)
    : AbstractConfigurationWidget (parent) {
    placeUIElements (getBaseLayout ());
  } 


void ColorConfigurationWidget::placeUIElements (QLayout * baseLayout) {
  assert (baseLayout != NULL);
  
  setTitle (COLOR_TITLE);

  chosenColor = DEFAULT_COLOR;

  QLabel * colorLabel = new QLabel ("Current color : ", this);
  colorChooserButton = createColorChooserButton ("", chosenColor);
  connect (colorChooserButton, SIGNAL (clicked()), 
	   this, SLOT (appearColorDialog())); 

  QHBoxLayout * colorButtonHBoxLayout = new QHBoxLayout ();
  colorButtonHBoxLayout->setMargin (COLOR_MARGIN);
  colorButtonHBoxLayout->setSpacing (TEX_SPACING);
  colorButtonHBoxLayout->addWidget (colorLabel);
  colorButtonHBoxLayout->addWidget (colorChooserButton);

  QVBoxLayout * colorVBoxLayout = new QVBoxLayout (this);
  colorVBoxLayout->setMargin (TEX_MARGIN);
  colorVBoxLayout->addLayout (baseLayout);
  colorVBoxLayout->addLayout (colorButtonHBoxLayout);
  colorVBoxLayout->activate (); 
}

// ********
// Services
// ********

TextureInterface * ColorConfigurationWidget::createTexture () const {
  return new SimpleColorTexture (getCurrentTextureName (), chosenColor);
}

QPixmap ColorConfigurationWidget::createPixmap () const {
  QPixmap pix (TEX_ICON_WIDTH, TEX_ICON_HEIGHT);
  pix.fill (chosenColor);
  return pix;
}

void ColorConfigurationWidget::conformTo (SimpleColorTexture * tex) {
  assert (tex != NULL);
  QString name = tex->getName ();
  
  chosenColor = tex->getColor ();
  updateButtonColor (colorChooserButton, chosenColor);
  setCurrentTextureName (name);
}

// *****
// Slots
// *****

void ColorConfigurationWidget::appearColorDialog () {
  selectColor (colorChooserButton, chosenColor);
}
